<?php
/**
 * FoxRate Currency Exchange Rate Processor
 *
 * @package blesta
 * @subpackage blesta.components.exchange_rates.foxrate
 * @copyright Copyright (c) 2010, Phillips Data, Inc.
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/ Blesta
 */
class Foxrate extends ExchangeRate {

	/**
	 * @var string The URL to the API
	 */
	private static $url = "http://foxrate.org/rpc/";
	/**
	 * @var int The maximum number of seconds to wait for a response
	 */
	private static $timeout = 30;

	/**
	 * Initializes the exchange rate processor
	 *
	 * @param Http $Http The Http component to more easily facilitate HTTP requests to fetch data
	 */
	public function __construct(Http $Http) {
		$this->Http = $Http;
	}
	
	/**
	 * Fetches the exchange rate from currency A to currency B using the given amount
	 *
	 * @param string $currency_from The ISO 4217 currency code to convert from
	 * @param string $currency_to The ISO 4217 currency code to convert to
	 * @param float $amount The amount to convert
	 * @return mixed (boolean) false on error or an array containing the exchange rate information including:
	 * 	-rate
	 * 	-updated The date/time of the last update in YYYY-MM-DD HH:MM:SS format in UTC time
	 */
	public function getRate($currency_from, $currency_to, $amount=1.0) {
		// Load the XML helper if not already loaded
		if (!isset($this->Xml))
			Loader::loadHelpers($this, array("Xml"));
			
		// Load Date helper if not set. Foxrate pulls data from OANDA, corp, which is based out of Toronto, Canada so we assume that timezone, but we need it in UTC
		if (!isset($this->Date))
			Loader::loadHelpers($this, array("Date"=>array(null, "America/Toronto", "UTC")));
			
		$data = array(
			'methodCall'=>array(
				'methodName'=>"foxrate.currencyConvert",
				'params'=>array(
					'param'=>array(
						array('value'=>array('string'=>$currency_from)),
						array('value'=>array('string'=>$currency_to)),
						array('value'=>array('double'=>$amount))
					)
				)
			)
		);
		
		$xml = $this->Xml->makeXml($data);
		
		// Send text/xml
		$this->Http->open();
		$this->Http->setHeader("Content-Type: text/xml");
		$this->Http->setTimeout(self::$timeout);
		
		// Submit the requests
		$response = $this->Http->post(self::$url, $xml);
		$this->Http->close();
		
		try {
			$response_obj = simplexml_load_string($response, "SimpleXMLElement", LIBXML_NOWARNING);
			$rate = array();
			
			if (isset($response_obj->params->param->value->struct->member)) {

				foreach ($response_obj->params->param->value->struct->member as $member) {
					$index = "";
					if ($member->name == "amount")
						$index = "rate";
					elseif ($member->name == "message")
						$index = "updated";
					else
						continue;
						
					foreach ((array)$member->value as $type => $value)
						$rate[$index] = str_replace("\"", "", (string)$value);
				}
			}
			
			if (!empty($rate)) {
				if ($rate['updated'] == "cached")
					$rate['updated'] = $this->Date->format("Y-m-d H:i:s");
				else
					$rate['updated'] = $this->Date->format("Y-m-d H:i:s", $rate['updated']);
				
				return $rate;
			}
		}
		catch (Exception $e) {
			// Error parsing XML
		}
		
		return false;
	}
}
?>